<?php

namespace LakeDrops\Component\Dotenv;

use Composer\IO\IOInterface;
use LakeDrops\Component\Composer\Utils;
use Symfony\Component\Dotenv\Dotenv as SymfonyDotenv;

/**
 * Manages .env files.
 */
final class Dotenv {

  /**
   * Name of the plugin "owning" this instance, used as a prefix.
   *
   * @var string
   */
  private $name;

  /**
   * The input-output object of the composer session.
   *
   * @var \Composer\IO\IOInterface
   */
  private $io;

  /**
   * The Symfony Dotenv object.
   *
   * @var \Symfony\Component\Dotenv\Dotenv
   */
  private $dotenv;

  /**
   * Dotenv constructor.
   *
   * @param string $name
   *   Name of the plugin "owning" this instance, used as a prefix.
   * @param \Composer\IO\IOInterface $io
   *   The input-output object of the composer session.
   */
  public function __construct($name, IOInterface $io) {
    $this->name = strtoupper($name);
    $this->io = $io;
    $this->dotenv = new SymfonyDotenv();
    $this->load();
  }

  /**
   * Receive a value from the environment with the project name as prefix.
   *
   * First of all, it prefixes the key with the plugin name and converts the
   * result into upper case. Then looking for the value with PHP's getenv and if
   * nothing found, it prompts the user for input if the session is interactive.
   * Finally, if the value is still not available, the default value is being
   * used.
   *
   * @param string $key
   *   The key of the environment variable.
   * @param string $prompt
   *   The text being displayed when prompting the user.
   * @param string $default
   *   The default value.
   *
   * @return string
   *   The value.
   */
  public function receive($key, $prompt, $default = '') {
    return $this->receiveGlobal($this->name . '_' . $key, $prompt, $default);
  }

  /**
   * Receive a value from the environment.
   *
   * @param string $key
   *   The key of the environment variable.
   * @param string $prompt
   *   The text being displayed when prompting the user.
   * @param string $default
   *   The default value.
   *
   * @return string
   *   The value.
   */
  public function receiveGlobal($key, $prompt, $default = '') {
    $key = strtoupper($key);
    $value = getenv($key);
    if (empty($value)) {
      if ($this->io->isInteractive()) {
        $value = $this->io->ask($prompt . ': ');
      }
      /** @noinspection NotOptimalIfConditionsInspection */
      if (empty($value)) {
        $value = $default;
      }
    }
    $this->put($key, $value);
    return $value;
  }

  /**
   * Set an environment variable.
   *
   * Sets the environment variable into a .env file only if there isn't an
   * environment variable present yet. The .env file will also be added to the
   * .gitignore file.
   *
   * @param string $key
   *   The name of the environment variable.
   * @param string $value
   *   The value of the environment variable.
   * @param bool $overwrite
   */
  public function put($key, $value, $overwrite = FALSE) {
    $envContent = file_exists('.env') ? file_get_contents('.env') : '';
    $data = $this->dotenv->parse($envContent);
    $key = strtoupper($key);
    if (!$overwrite && isset($data[$key])) {
      return;
    }
    $data[$key] = $value;
    $envContent = '';
    foreach ($data as $k => $v) {
      $envContent .= $k . '=' . $v . PHP_EOL;
    }
    file_put_contents('.env', $envContent);
    $this->load();
    Utils::gitIgnore('.env');
  }

  /**
   * Replace environment variables in keys and values of a given array.
   *
   * Credit: https://github.com/composer/composer/pull/5837
   *
   * @param mixed $data
   *   The keyed array in which to replace environment variables.
   *
   * @return array
   *   The same data with environment variables being replaced.
   */
  public function replaceEnvironmentVariables($data) {
    array_walk_recursive($data, array($this, 'findEnvironmentVariables'));
    return $data;
  }

  /**
   * Identify and replace environment variables in a key pair
   *
   * @param string $item
   * @param string $key
   */
  private function findEnvironmentVariables(&$item, &$key) {
    $pattern = '@\{\$env:([A-Za-z0-9_]*)\}@i';
    preg_match_all($pattern, $item, $item_matches);
    preg_match_all($pattern, $key, $key_matches);

    if (!empty($item_matches[1])) {
      $item = $this->replaceItemEnvironmentVariables($item, $item_matches[1]);
    }
    if (!empty($key_matches[1])) {
      $key = $this->replaceItemEnvironmentVariables($key, $key_matches[1]);
    }
  }

  /**
   * Replace environment variables in a string
   *
   * @param string $item
   * @param array $matches
   *
   * @return string
   */
  private function replaceItemEnvironmentVariables($item, $matches) {
    foreach ($matches as $var) {
      $value = getenv($var);
      if (!$value) {
        $value = '';
      }
      $item = str_replace( '{$env:' . $var . '}', $value, $item );
    }
    return $item;
  }

  /**
   * Load the local .env file.
   */
  private function load() {
    try {
      $this->dotenv->load('.env');
    }
    catch (\Exception $ex) {
      // Ignore.
    }
  }

}
